Texture texture;

struct Vertex
{
	float u;
	float v;
};

Vertex MakeVertex(float u, float v)
{
	Vertex a;
	a.u=u;
	a.v=v;
	return a;
}

struct Triangle
{
	Vertex tc1[3];
	Vertex tc2[3];
};

Vertex* verts;
Vertex* verts2;
Triangle* tris;
int numverts;
int numtris;

bool ReadLine(FILE* file, char* dest)
{
	char c=' ';
	fread(&c, 1, 1, file);
	while(c!='\n' && !feof(file))
	{
		*dest++=c;
		fread(&c, 1, 1, file);
	}
	*dest++='\0';
	if(feof(file))
		return false;
	return true;
}

/*

salient bits of the OBJ file format:

v 1.0 1.0 1.0    - vertex position, automatic index numbering
vt 1.0 1.0       - texture coordinate pair, automatic index numbering
f 0/0 1/1 2/5    - triangular face, with indices to position/texcoord for each corner, can be different indices for each

*/

void clean_face(char* buffer) // strip unwanted characters to make "45/254 27/9" become "   254    9"
{
	int l=strlen(buffer);
	int mode=0;
	for(int i=0;i<l;i++)
	{
		char c=buffer[i];
		if(c==' ')
			mode=0;
		if(mode==0 && c>='0' && c<='9')
			mode=1;
		if(c=='/')
		{
			mode=2;
			buffer[i]=' ';
		}
		if(mode!=2)
			buffer[i]=' ';
	}
}

bool LoadOBJ(char* filename1, char* filename2)
{
	// open first file and read texture coordinates + triangles

	FILE* file=fopen(filename1, "r");
	if(!file)
		return false;
	char line[256];
	numtris=0;
	numverts=0;
	while(ReadLine(file, line))
	{
		char cmd[256];
		strcpy(cmd, "XXX");
		sscanf(line, "%s ", cmd);
		if(strcmp(cmd, "v")==0) // vertex
			numverts++;
		if(strcmp(cmd, "f")==0) // face/triangle
			numtris++;
	}

	verts=new Vertex[numverts*2];
	tris=new Triangle[numtris];
	int cur_vert=0;
	int cur_tri=0;
	fseek(file, 0, 0); // reset file
	while(ReadLine(file, line))
	{
		char cmd[256];
		strcpy(cmd, "XXX");
		sscanf(line, "%s ", cmd);
		if(strcmp(cmd, "vt")==0) // texcoord
		{
			float u, v;
			sscanf(line, "%s %f %f", cmd, &u, &v);
			verts[cur_vert].u=u;
			verts[cur_vert].v=1.0f-v;
			cur_vert++;
		}
	}
	fseek(file, 0, 0); // reset file
	int fi=0;
	while(ReadLine(file, line))
	{
		char cmd[256];
		strcpy(cmd, "XXX");
		sscanf(line, "%s ", cmd);
		if(strcmp(cmd, "f")==0) // face/triangle
		{
			int a=-1, b=-1, c=-1;
			clean_face(line);
			sscanf(line, "%i %i %i", &a, &b, &c);
			tris[cur_tri].tc1[0]=MakeVertex(verts[a-1].u, verts[a-1].v);
			tris[cur_tri].tc1[1]=MakeVertex(verts[b-1].u, verts[b-1].v);
			tris[cur_tri].tc1[2]=MakeVertex(verts[c-1].u, verts[c-1].v);
			cur_tri++;
		}
	}
	fclose(file);

	// open second file and read another set of texture coordinates

	file=fopen(filename2, "r");
	if(!file)
		return false;

	verts2=new Vertex[numverts*2]; // could be more of these (or less)

	cur_vert=0;
	fseek(file, 0, 0); // reset file
	while(ReadLine(file, line))
	{
		char cmd[256];
		strcpy(cmd, "XXX");
		sscanf(line, "%s ", cmd);
		if(strcmp(cmd, "vt")==0) // texcoord
		{
			float u, v;
			sscanf(line, "%s %f %f", cmd, &u, &v);
			verts2[cur_vert].u=u;
			verts2[cur_vert].v=1.0f-v;
			cur_vert++;
		}
	}
	fseek(file, 0, 0); // reset file
	cur_tri=0;
	while(ReadLine(file, line))
	{
		char cmd[256];
		strcpy(cmd, "XXX");
		sscanf(line, "%s ", cmd);
		if(strcmp(cmd, "f")==0) // face/triangle
		{
			int a=-1, b=-1, c=-1;
			clean_face(line);
			sscanf(line, "%i %i %i", &a, &b, &c);
			tris[cur_tri].tc2[0]=MakeVertex(verts2[a-1].u, verts2[a-1].v);
			tris[cur_tri].tc2[1]=MakeVertex(verts2[b-1].u, verts2[b-1].v);
			tris[cur_tri].tc2[2]=MakeVertex(verts2[c-1].u, verts2[c-1].v);
			cur_tri++;
		}
	}
	fclose(file);

	delete[] verts2;
	
	return true;
}

void Render()
{
	glClearColor(0.0f, 0.0f, 1.0f, 0.0f);
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT|GL_STENCIL_BUFFER_BIT);

	SetOrtho(true);

	glDisable(GL_DEPTH_TEST);
	glDisable(GL_CULL_FACE);
	glLoadIdentity();
	glDisable(GL_BLEND);

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, texture.getHandle());
	glColor4f(1,1,1,1);

	// draw each triangle using the texture coordinates from the first mesh for uv mapping,
	//  and the texture coordinates from the second mesh as vertex positions

	// scale to fit within pixel size of texture image

	glBegin(GL_TRIANGLES);
	for(int i=0;i<numtris;i++)
	{
		Triangle& t=tris[i];
		for(int j=0;j<3;j++)
		{
			glTexCoord2f(t.tc1[j].u, t.tc1[j].v);
			glVertex3f(20+t.tc2[j].u*texture.width, 20+t.tc2[j].v*texture.height, 0);
		}
	}
	glEnd();
}

void Init()
{
	texture.LoadPNG("texture.png", TEXTURE_NOMIPMAPS);

	LoadOBJ("mesh1.obj", "mesh2.obj");
}

